/**
 * test-webid
 * Copyright (C) 2011 Thomas Bergwinkl
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bergnet.webid.commons;

import java.awt.EventQueue;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.filechooser.FileFilter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.border.EtchedBorder;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;


public class WebIDToolSwingGUI implements ActionListener {
	private JFrame frameWebIDTool;
	private JButton buttonCreateWebid = new JButton("Create WebID");
	private JButton buttonSaveFoaf = new JButton("Save FOAF");
	private JButton buttonOpenPkcs12 = new JButton("Open PKCS12");
	private JButton buttonSavePkcs12 = new JButton("Save PKCS12");
	private JButton buttonClear = new JButton("Clear");
	private JTextField textFieldWebID = new JTextField();
	private JTextPane textPaneModulus = new JTextPane();
	private JTextField textFieldPublicExponent = new JTextField();
	private JComboBox comboBoxKeySize = new JComboBox();
	private JTextField textFieldValidFrom;
	private JTextField textFieldValidTill;
	private JComboBox comboBoxFoafFormat = new JComboBox();

	private WebID webid = null;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					WebIDToolSwingGUI window = new WebIDToolSwingGUI();
					window.frameWebIDTool.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public WebIDToolSwingGUI() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frameWebIDTool = new JFrame();
		frameWebIDTool.setResizable(false);
		frameWebIDTool.setTitle("WebIDTool");
		frameWebIDTool.setBounds(100, 100, 469, 746);
		frameWebIDTool.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frameWebIDTool.getContentPane().setLayout(null);

		JLayeredPane panelButtons = new JLayeredPane();
		panelButtons.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		panelButtons.setBounds(10, 11, 440, 84);

		frameWebIDTool.getContentPane().add(panelButtons);
		panelButtons.setLayout(null);
		buttonCreateWebid.setBounds(10, 11, 130, 23);
		panelButtons.add(buttonCreateWebid);
		
		buttonCreateWebid.addActionListener(this);
		buttonSaveFoaf.setBounds(150, 11, 130, 23);
		panelButtons.add(buttonSaveFoaf);
		buttonSaveFoaf.addActionListener(this);
		buttonOpenPkcs12.setBounds(10, 45, 130, 23);
		panelButtons.add(buttonOpenPkcs12);
		buttonOpenPkcs12.addActionListener(this);
		buttonSavePkcs12.setBounds(150, 45, 130, 23);
		panelButtons.add(buttonSavePkcs12);

		buttonClear.setBounds(290, 45, 130, 23);
		panelButtons.add(buttonClear);
		buttonSavePkcs12.addActionListener(this);
		buttonClear.addActionListener(this);
		
		JLayeredPane panePublicKey = new JLayeredPane();
		panePublicKey.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		panePublicKey.setBounds(10, 342, 440, 365);
		frameWebIDTool.getContentPane().add(panePublicKey);
		panePublicKey.setLayout(null);
				
		JLabel labelModulus = new JLabel("Modulus");
		labelModulus.setBounds(10, 11, 100, 30);
		panePublicKey.add(labelModulus);
		textPaneModulus.setEditable(false);
		textPaneModulus.setBounds(120, 11, 300, 302);
		panePublicKey.add(textPaneModulus);
		
		JLabel labelPublicExponent = new JLabel("Public Exponent");
		labelPublicExponent.setBounds(10, 324, 100, 30);
		panePublicKey.add(labelPublicExponent);
		textFieldPublicExponent.setEditable(false);
		textFieldPublicExponent.setBounds(120, 324, 300, 30);
		panePublicKey.add(textFieldPublicExponent);
		
		JLayeredPane paneWebID = new JLayeredPane();
		paneWebID.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		paneWebID.setBounds(10, 277, 440, 54);
		frameWebIDTool.getContentPane().add(paneWebID);
		
		JLabel labelWebid = new JLabel("WebID");
		labelWebid.setBounds(10, 11, 100, 30);
		paneWebID.add(labelWebid);
		textFieldWebID.setBounds(120, 11, 300, 30);
		paneWebID.add(textFieldWebID);
		textFieldWebID.setText("http://");
		textFieldWebID.setColumns(10);

		JLayeredPane paneKey = new JLayeredPane();
		paneKey.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		paneKey.setBounds(10, 106, 167, 54);
		frameWebIDTool.getContentPane().add(paneKey);

		JLabel labelKeySize = new JLabel("Key Size");
		labelKeySize.setBounds(10, 11, 70, 30);

		paneKey.add(labelKeySize);

		comboBoxKeySize.setModel(new DefaultComboBoxModel(new String[] {"1024", "2048", "4096"}));
		comboBoxKeySize.setBounds(90, 11, 65, 30);
		paneKey.add(comboBoxKeySize);

		JLayeredPane paneValidDate = new JLayeredPane();
		paneValidDate.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		paneValidDate.setBounds(10, 171, 440, 95);
		frameWebIDTool.getContentPane().add(paneValidDate);

		JLabel labelValidFrom = new JLabel("Valid from");
		labelValidFrom.setBounds(10, 11, 100, 30);		
		paneValidDate.add(labelValidFrom);

		textFieldValidFrom = new JTextField();
		textFieldValidFrom.setColumns(10);
		textFieldValidFrom.setBounds(120, 11, 300, 30);
		paneValidDate.add(textFieldValidFrom);
		
		JLabel labelValidTill = new JLabel("Valid Till");
		labelValidTill.setBounds(10, 52, 100, 30);
		paneValidDate.add(labelValidTill);
		
		textFieldValidTill = new JTextField();
		textFieldValidTill.setColumns(10);
		textFieldValidTill.setBounds(120, 52, 300, 30);
		paneValidDate.add(textFieldValidTill);
		
		JLayeredPane paneFoafFormat = new JLayeredPane();
		paneFoafFormat.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		paneFoafFormat.setBounds(187, 106, 263, 54);
		frameWebIDTool.getContentPane().add(paneFoafFormat);
		
		JLabel labelFoafFormat = new JLabel("FOAF Format");
		labelFoafFormat.setBounds(10, 11, 80, 30);
		paneFoafFormat.add(labelFoafFormat);

		comboBoxFoafFormat.setModel(new DefaultComboBoxModel(new String[] {"application/rdf+xml", "text/turtle"}));
		comboBoxFoafFormat.setBounds(93, 11, 160, 30);
		paneFoafFormat.add(comboBoxFoafFormat);

		setWebId(null);
	}

	public void actionPerformed(ActionEvent event) {
		try {
			if(event.getSource() == buttonCreateWebid)
				createWebid();
			else if(event.getSource() == buttonSaveFoaf)
				saveFoaf();
			else if(event.getSource() == buttonOpenPkcs12)
				openPkcs12();
			else if(event.getSource() == buttonSavePkcs12)
				savePkcs12();
			else if(event.getSource() == buttonClear)
				setWebId(null);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void createWebid() throws Exception {
		int keySize = Integer.parseInt(comboBoxKeySize.getSelectedItem().toString(), 10);

		String[] uris = textFieldWebID.getText().split(",");

		Date validFrom = getDateFormat().parse(textFieldValidFrom.getText());
		Date validTill = getDateFormat().parse(textFieldValidTill.getText());

		setWebId(WebID.generate(keySize, uris, validFrom, validTill));
	}

	public void saveFoaf() throws Exception {
		FileFilter filter = new FileExtensionFilter(new String[] {"rdf", "ttl"});
		JFileChooser fileChooser = new JFileChooser();

		fileChooser.setFileFilter(filter);

		if(fileChooser.showSaveDialog(frameWebIDTool) == JFileChooser.APPROVE_OPTION) {
			String fileName = fileChooser.getSelectedFile().getAbsolutePath();
			String format = comboBoxFoafFormat.getSelectedItem().toString();

			WebIDTool.writeFoafFile(webid, fileName, format);
		}
	}

	public void openPkcs12() throws Exception {
		FileFilter filter = new FileExtensionFilter(new String[] {"p12"});
		JFileChooser fileChooser = new JFileChooser();

		fileChooser.setFileFilter(filter);

		if(fileChooser.showOpenDialog(frameWebIDTool) == JFileChooser.APPROVE_OPTION) {
			String fileName = fileChooser.getSelectedFile().getAbsolutePath();
			String password = passwordPopup();

			setWebId(CryptoUtils.readPkcs12File(fileName, null, password));
		}
	}

	public void savePkcs12() throws Exception {
		FileFilter filter = new FileExtensionFilter(new String[] {"p12"});
		JFileChooser fileChooser = new JFileChooser();

		fileChooser.setFileFilter(filter);

		if(fileChooser.showSaveDialog(frameWebIDTool) == JFileChooser.APPROVE_OPTION) {
			String fileName = fileChooser.getSelectedFile().getAbsolutePath();
			String password = passwordPopup();

			CryptoUtils.writePkcs12File(webid, fileName, "webid", password);
		}
	}

	public void setWebId(WebID webid) {
		this.webid = webid;

		if(webid == null) {
			// WebID TextField
			textFieldWebID.setEditable(true);
			textFieldWebID.setText("http://");

			// Valid From TextField
			textFieldValidFrom.setEditable(true);
			Date now = new Date();
			textFieldValidFrom.setText(getDateFormat().format(now));

			// Valid Till TextField
			textFieldValidTill.setEditable(true);
			textFieldValidTill.setText(getDateFormat().format(WebID.createDeltaDate(365)));

			// Key Size ComboBox
			comboBoxKeySize.setEditable(true);
			comboBoxKeySize.setSelectedIndex(1);

			// Modulus TextPane
			textPaneModulus.setText(null);

			// Public Exponent TextField
			textFieldPublicExponent.setText(null);
		} else {
			// WebID TextField
			textFieldWebID.setEditable(false);
			String uriString = null;
			for(String uri : this.webid.getUris()) {
				if(uriString == null)
					uriString = uri;
				else
					uriString += "," + uri;
			}
			textFieldWebID.setText(uriString);

			// Valid From TextField
			textFieldValidFrom.setEditable(false);
			textFieldValidFrom.setText(getDateFormat().format(this.webid.getCertificate().getNotBefore()));

			// Valid Till TextField
			textFieldValidTill.setEditable(false);
			textFieldValidTill.setText(getDateFormat().format(this.webid.getCertificate().getNotAfter()));

			// Key Size ComboBox
			comboBoxKeySize.setEditable(false);
			int keySizeIndex = (int)Math.floor(Math.log(webid.getModulus().bitLength())/Math.log(2.0)-9.5);
			comboBoxKeySize.setSelectedIndex(keySizeIndex);

			// Modulus TextPane
			textPaneModulus.setText(this.webid.getModulus().toString(16));

			// Public Exponent TextField
			textFieldPublicExponent.setText(this.webid.getPublicExponent().toString(10));
		}
	}

	public static String passwordPopup() {
		JPasswordField passwordField = new JPasswordField();
		passwordField.setEchoChar('#');

		JOptionPane.showMessageDialog(null, passwordField, "Enter password", JOptionPane.OK_OPTION);

		return new String(passwordField.getPassword());
	}

	public static DateFormat getDateFormat() {
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
	}

	private class FileExtensionFilter extends FileFilter {
		private String[] extensions;

		public FileExtensionFilter(String[] extensions) {
			this. extensions = extensions;
		}

		public boolean accept(File file) {
			if (file.isDirectory())
				return true;

			String currentExtension = getFileExtension(file.getName());

		    if(currentExtension != null) {
		    	for(String extension : this.extensions) {
		    		if(extension.equals(currentExtension))
		    			return true;
		    		else
		    			return false;
		    	}
		    }

		    return false;
		}
	
		private String getFileExtension(String fileName) {
			String extension = null;
			int i = fileName.lastIndexOf('.');

			if(i>0 &&  i<fileName.length()-1)
				extension = fileName.substring(i+1).toLowerCase();

			return extension;
		}

		public String getDescription() {
			return "FileExtensionFilter";
		}
	}
}
